# Generated by roxygen2: do not edit by hand

S3method(fitted,stl)
S3method(fitted,stlplus)
S3method(plot,stlplus)
S3method(predict,stl)
S3method(predict,stlplus)
S3method(remainder,stl)
S3method(remainder,stlplus)
S3method(seasonal,stl)
S3method(seasonal,stlplus)
S3method(stlplus,default)
S3method(stlplus,ts)
S3method(stlplus,zoo)
S3method(time,stl)
S3method(time,stlplus)
S3method(trend,stl)
S3method(trend,stlplus)
export(fc)
export(getraw)
export(plot_cycle)
export(plot_rembycycle)
export(plot_seasonal)
export(plot_trend)
export(remainder)
export(seasonal)
export(stlplus)
export(trend)
importFrom(Rcpp,evalCpp)
importFrom(lattice,panel.abline)
importFrom(lattice,panel.loess)
importFrom(lattice,panel.segments)
importFrom(lattice,panel.xyplot)
importFrom(lattice,xyplot)
importFrom(stats,frequency)
importFrom(stats,loess)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,time)
importFrom(stats,weighted.mean)
importFrom(utils,head)
importFrom(utils,stack)
importFrom(utils,tail)
importFrom(yaImpute,ann)
useDynLib(stlplus, .registration = TRUE)
