# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

c_interp <- function(m, fits, slopes, at) {
    .Call(`_stlplus_c_interp`, m, fits, slopes, at)
}

c_loess <- function(xx, yy, degree, span, ww, m, l_idx, max_dist) {
    .Call(`_stlplus_c_loess`, xx, yy, degree, span, ww, m, l_idx, max_dist)
}

c_ma <- function(x, n_p) {
    .Call(`_stlplus_c_ma`, x, n_p)
}

